#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const path = __importStar(require("path"));
const pacote_1 = __importDefault(require("pacote"));
const tar_stream_1 = __importDefault(require("tar-stream"));
const zlib_1 = __importDefault(require("zlib"));
const stream_1 = require("stream");
const util_1 = require("util");
const chalk_1 = __importDefault(require("chalk"));
const pipelineAsync = (0, util_1.promisify)(stream_1.pipeline);
async function getDependencySizes(packageJsonPath) {
    const packageJson = JSON.parse(await fs_1.promises.readFile(packageJsonPath, "utf-8"));
    const dependencies = packageJson.dependencies || {};
    let totalUnpackedSize = 0;
    console.log(chalk_1.default.bold("Calculating unpacked sizes of dependencies...\n"));
    console.log(chalk_1.default.cyan.bold("Package Name") + "  |  " + chalk_1.default.cyan.bold("Unpacked Size"));
    console.log(chalk_1.default.cyan("-".repeat(40)));
    for (const [dep, version] of Object.entries(dependencies)) {
        try {
            const tarballStream = await pacote_1.default.tarball.stream(`${dep}@${version}`, (stream) => Promise.resolve(stream));
            const unpackedSize = await calculateTarballSize(tarballStream);
            totalUnpackedSize += unpackedSize;
            const sizeInKB = unpackedSize / 1024;
            const sizeInMB = sizeInKB / 1024;
            let sizeDisplay = `${sizeInKB.toFixed(2)} KB`;
            // Display in MB if the size is greater than 1024 KB
            if (sizeInMB >= 1) {
                sizeDisplay = `${sizeInMB.toFixed(2)} MB`;
            }
            // Colorize output based on size
            if (sizeInKB > 500) {
                console.log(chalk_1.default.red(`${dep.padEnd(20)} |  ${sizeDisplay}`));
            }
            else {
                console.log(chalk_1.default.green(`${dep.padEnd(20)} |  ${sizeDisplay}`));
            }
        }
        catch (error) {
            if (error instanceof Error) {
                console.error(chalk_1.default.yellow(`Failed to fetch size for ${dep}: ${error.message}`));
            }
            else {
                console.error(chalk_1.default.yellow(`Failed to fetch size for ${dep}: Unknown error`));
            }
        }
    }
    console.log(chalk_1.default.cyan("-".repeat(40)));
    const totalSizeInKB = totalUnpackedSize / 1024;
    const totalSizeInMB = totalSizeInKB / 1024;
    let totalSizeDisplay = `${totalSizeInKB.toFixed(2)} KB`;
    // Display total size in MB if greater than 1024 KB
    if (totalSizeInMB >= 1) {
        totalSizeDisplay = `${totalSizeInMB.toFixed(2)} MB`;
    }
    console.log(chalk_1.default.bold(`Total Unpacked Size: ${chalk_1.default.magenta.bold(totalSizeDisplay)}`));
}
async function calculateTarballSize(tarballStream) {
    const extract = tar_stream_1.default.extract();
    const gunzip = zlib_1.default.createGunzip();
    let totalSize = 0;
    extract.on("entry", (header, stream, next) => {
        totalSize += header.size || 0; // Accumulate the size of each entry
        stream.resume(); // Consume the stream
        stream.on("end", next);
    });
    // Decompress the tarball stream and pipe it to the tar extractor
    await pipelineAsync(tarballStream, gunzip, extract);
    return totalSize;
}
// Locate the nearest package.json and calculate dependency sizes
(async () => {
    const packageJsonPath = path.resolve(process.cwd(), "package.json");
    try {
        console.log(chalk_1.default.bold(`Found package.json at: ${packageJsonPath}\n`));
        await getDependencySizes(packageJsonPath);
    }
    catch (error) {
        if (error instanceof Error) {
            console.error(chalk_1.default.red(`Error: ${error.message}`));
        }
        else {
            console.error(chalk_1.default.red("Unknown error occurred."));
        }
    }
})();
